
require "Window"
 
local GuildAlias = {} 
local av = "1.1.0" -- used for the saved variables. This shall be increased when there are changes in the structure of saved data

function GuildAlias:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

	self.storedVars = {} -- saved variables

    return o
end

function GuildAlias:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"ChatLog",
	}
	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)

	Apollo.RegisterSlashCommand("iam", "ShowOptions", self, param)
	Apollo.RegisterSlashCommand("guildalias", "ShowOptions", self) -- option frame

	-- store channel types
	self.chat_commands = { -- we can only access the commands and abbreviation in the lang out client uses. But I for example prefer using /y when I want to yell... So we need a bit manual mapping
		--account whisper
		aw = 34,
		awhisper = 34,
		
		-- instance
		i = 32,
		instance = 32,
		
		-- party/group
		p = 7,
		party = 7,
		
		-- pvp
		pvp = 10,
		
		-- yell
		y = 5,
		yell = 5,
		
		-- say
		s = 4,
		say = 4,
		
		-- normal char whisper
		whisper = 6,
		t = 6, 
		tell = 6, 
		w = 6,
		
		-- guild
		guild = 15,
		g = 15,
		
		-- trade
		tr = 11,
		trade = 11,
		
		-- zone
		z = 9,
		zone = 9
	}
	
	-- store the localized abbreviation and command for the desired channel that shall get our prefix
	for k, v in pairs(ChatSystemLib.GetChannels()) do
		-- extend the manual table map with localized data
		self.chat_commands[v:GetCommand()] = v:GetType()
		self.chat_commands[v:GetAbbreviation()] = v:GetType()
	end
end

-- see https://forums.wildstar-online.com/forums/index.php?/topic/93286-saving-data-in-my-addon/
function GuildAlias:OnSave(eType)
	return self.storedVars 
end

function GuildAlias:OnRestore(eType, tSavedData)
	self.storedVars = tSavedData
end

function GuildAlias:ShowOptions()
	self.wndOpt = Apollo.LoadForm("GuildAlias.xml", "ConfigForm", nil, self)
	
	-- wow.. this definitely should be done with a loop:
	-- So: Name the Input suffix exactly like the enums for the channel
	-- write a loop which regexes out the suffix and reflect it onto a enum.. but.. is lua even able to do reflection? 
	if(self.storedVars[av]) then
		if(self.storedVars[av][GameLib.GetRealmName()]) then
			if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()]) then
				if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Say]) 			then self.wndOpt:FindChild("UIPlayerNameSay"):SetText(		self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Say]) end
				if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Yell]) 		then self.wndOpt:FindChild("UIPlayerNameYell"):SetText(		self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Yell]) end
				if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Party]) 		then self.wndOpt:FindChild("UIPlayerNameParty"):SetText(	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Party]) end
				if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Instance]) 	then self.wndOpt:FindChild("UIPlayerNameDungeon"):SetText(	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Instance]) end
				if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Trade]) 		then self.wndOpt:FindChild("UIPlayerNameTrade"):SetText(	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Trade]) end
				--if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Whisper]) then self.wndOpt:FindChild("UIPlayerNameWhisper"):SetText(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Whisper]) end
				--if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_WarParty]) then self.wndOpt:FindChild("UIPlayerNamePVP"):SetText(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_WarParty]) end
				if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Zone]) 		then self.wndOpt:FindChild("UIPlayerNameZone"):SetText(		self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Zone]) end
				if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Guild]) 		then self.wndOpt:FindChild("UIPlayerNameGuild"):SetText(	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Guild]) end
			end
		end
	end
	
	self.wndOpt:FindChild("playername"):SetText( GameLib.GetPlayerUnit():GetName() )
	self.wndOpt:FindChild("servername"):SetText( GameLib.GetRealmName() )

end

function GuildAlias:OnLoad()
	Apollo.GetPackage("Gemini:Hook-1.0").tPackage:Embed(self)
	
	self:RawHook(Apollo.GetAddon("ChatLog"), "OnChatInputReturn")
end



function GuildAlias:OnChatInputReturn(luaCaller, wndHandler, wndControl, strText)
	
	local channel_id = nil
	local cmd = nil
	if( string.match(strText, "\/%a*%s?") ~= nil ) then -- if we really got a cmd
		cmd = string.match(strText, "\/(%a*)%s?")
		channel_id = self.chat_commands[cmd]
	else
		if(luaCaller["channelLastChannel"]) then
			channel_id = luaCaller["channelLastChannel"]:GetType()
		else
			channel_id = ChatSystemLib.ChatChannel_Say -- default fallback
		end
	end
		
	local player_alias = nil
	if(self.storedVars[av]) then
		if(self.storedVars[av][GameLib.GetRealmName()]) then
			if(self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()]) then
				player_alias = self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][channel_id]
			end
		end
	end

	if(player_alias ~= nil and string.len(strText) > 0) then
		if ( cmd == nil ) then
			-- no command
			strText = player_alias .. strText
		else
			-- command in front of chat
			if(string.len(strText) > string.len(cmd)+1 ) then -- prevent strange behaviour for (example) /y<ENTER>
				-- 				the command 					 	the stored playername      the rest of the message 
				strText = string.sub(strText,0,string.len(cmd)+2) .. player_alias .. string.sub(strText, string.len(cmd)+3 ) -- +2 because of the stripped / and the whitespace at the end of the cmd
			end
		end
	end
	
	-- call original
	self.hooks[Apollo.GetAddon("ChatLog")].OnChatInputReturn(luaCaller, wndHandler, wndControl, strText)
end

function GuildAlias:OnOK( wndHandler, wndControl, eMouseButton )

	-- make sure the basic structure is up before we try to save anything
	if (self.storedVars[av] == nil) then
		self.storedVars[av] = {}
	end
	if (self.storedVars[av][GameLib.GetRealmName()] == nil) then
		self.storedVars[av][GameLib.GetRealmName()] = {}
	end
	if (self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()] == nil) then
		self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()] = {}
	end
	
	-- save the configuration to that structure
	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Say] = self.wndOpt:FindChild("UIPlayerNameSay"):GetText()
	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Yell] = self.wndOpt:FindChild("UIPlayerNameYell"):GetText()
	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Party] = self.wndOpt:FindChild("UIPlayerNameParty"):GetText()
	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Instance] = self.wndOpt:FindChild("UIPlayerNameDungeon"):GetText()
	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Trade] = self.wndOpt:FindChild("UIPlayerNameTrade"):GetText()
	--self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Whisper] = self.wndOpt:FindChild("UIPlayerNameWhisper"):GetText()
	--self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_WarParty] = self.wndOpt:FindChild("UIPlayerNamePVP"):GetText()
	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Zone] = self.wndOpt:FindChild("UIPlayerNameZone"):GetText()
	self.storedVars[av][GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()][ChatSystemLib.ChatChannel_Guild] = self.wndOpt:FindChild("UIPlayerNameGuild"):GetText()
	
	self.wndOpt:Show(false)
	Print("GuildAlias: Your settings were updated")
end

function GuildAlias:OnCancel( wndHandler, wndControl, eMouseButton )
	self.wndOpt:Show(false)
end

-----------------------------------------------------------------------------------------------
-- GuildAlias Instance
-----------------------------------------------------------------------------------------------
local GuildAliasInst = GuildAlias:new()
GuildAliasInst:Init()
